@@ -37,6 +37,27 @@
  * through the #ClutterScript definition format, but it comes with a
  * convenience C API.
  *
+ * <refsect2 id="ClutterAnimator-key-frames">
+ *   <title>Key Frames</title>
+ *   <para>Every animation handled by a #ClutterAnimator can be
+ *   described in terms of "key frames". For each #GObject property
+ *   there can be multiple key frames, each one defined by the end
+ *   value for the property to be computed starting from the current
+ *   value to a specific point in time, using a given easing
+ *   mode.</para>
+ *   <para>The point in time is defined using a value representing
+ *   the progress in the normalized interval of [ 0, 1 ]. This maps
+ *   the value returned by clutter_timeline_get_duration().</para>
+ *   <figure id="easing-modes">
+ *     <title>Key Frames</title>
+ *     <graphic fileref="animator-key-frames.png" format="PNG"/>
+ *   </figure>
+ *   <para>In the image above the duration of the animation is
+ *   represented by the blue line. Each key frame is the white dot,
+ *   along with its progress. The red line represents the computed
+ *   function of time given the easing mode.</para>
+ * </refsect2>
+ *
  * <refsect2 id="ClutterAnimator-script">
  *   <title>ClutterAnimator description for #ClutterScript</title>
  *   <para>#ClutterAnimator defines a custom "properties" property
@@ -160,7 +181,8 @@ enum
 {
   PROP_0,
 
-  PROP_DURATION
+  PROP_DURATION,
+  PROP_TIMELINE
 };
 
 static void clutter_scriptable_init (ClutterScriptableIface *iface);
@@ -173,9 +195,11 @@ G_DEFINE_TYPE_WITH_CODE (ClutterAnimator,
 /**
  * clutter_animator_new:
  *
- * Create a new #ClutterAnimator instance.
+ * Creates a new #ClutterAnimator instance
  *
- * Returns: a new #ClutterAnimator.
+ * Return value: a new #ClutterAnimator.
+ *
+ * Since: 1.2
  */
 ClutterAnimator *
 clutter_animator_new (void)
@@ -457,36 +481,47 @@ animation_animator_ensure_animator (ClutterAnimator *animator,
           initial = g_list_find_custom (key_animator->current->next,
                                         key,
                                         sort_actor_prop_func);
-          g_assert (initial != NULL);
 
-          initial_key = initial->data;
-
-          clutter_interval_set_initial_value (key_animator->interval,
-                                              &initial_key->value);
-          key_animator->current = initial;
-          key_animator->start = initial_key->progress;
-
-          next = g_list_find_custom (initial->next,
-                                     key,
-                                     sort_actor_prop_func);
-          if (next)
+          if (initial)
             {
-              next_key = next->data;
+              initial_key = initial->data;
 
-              key_animator->end = next_key->progress;
+              clutter_interval_set_initial_value (key_animator->interval,
+                                                  &initial_key->value);
+              key_animator->current = initial;
+              key_animator->start = initial_key->progress;
+
+              next = g_list_find_custom (initial->next,
+                                         key,
+                                         sort_actor_prop_func);
+              if (next)
+                {
+                  next_key = next->data;
+
+                  key_animator->end = next_key->progress;
+                }
+              else
+                {
+                  next_key = initial_key;
+
+                  key_animator->end = 1.0;
+                }
+
+              clutter_interval_set_final_value (key_animator->interval,
+                                                &next_key->value);
+
+              if ((clutter_alpha_get_mode (key_animator->alpha) != next_key->mode))
+                clutter_alpha_set_mode (key_animator->alpha, next_key->mode);
             }
-          else
+          else /* no relevant interval */
             {
-              next_key = initial_key;
-
-              key_animator->end = 1.0;
+              ClutterAnimatorKey *current_key = key_animator->current->data;
+              clutter_interval_set_initial_value (key_animator->interval,
+                                                  &current_key->value);
+              clutter_interval_set_final_value (key_animator->interval,
+                                                &current_key->value);
+              break;
             }
-
-          clutter_interval_set_final_value (key_animator->interval,
-                                            &next_key->value);
-
-          if ((clutter_alpha_get_mode (key_animator->alpha) != next_key->mode))
-            clutter_alpha_set_mode (key_animator->alpha, next_key->mode);
         }
     }
   else if (progress < key_animator->start)
@@ -500,33 +535,37 @@ animation_animator_ensure_animator (ClutterAnimator *animator,
           initial = list_find_custom_reverse (key_animator->current->prev,
                                               key,
                                               sort_actor_prop_func);
-          g_assert (initial != NULL);
 
-          initial_key = initial->data;
-
-          clutter_interval_set_initial_value (key_animator->interval,
-                                              &initial_key->value);
-          key_animator->current = initial;
-          key_animator->end = key_animator->start;
-          key_animator->start = initial_key->progress;
-
-          if (old)
+          if (initial)
             {
-              next_key = old->data;
+              initial_key = initial->data;
 
-              key_animator->end = next_key->progress;
+              clutter_interval_set_initial_value (key_animator->interval,
+                                                  &initial_key->value);
+              key_animator->current = initial;
+              key_animator->end = key_animator->start;
+              key_animator->start = initial_key->progress;
+
+              if (old)
+                {
+                  next_key = old->data;
+
+                  key_animator->end = next_key->progress;
+                }
+              else
+                {
+                  next_key = initial_key;
+
+                  key_animator->end = 1.0;
+                }
+
+              clutter_interval_set_final_value (key_animator->interval,
+                                                &next_key->value);
+              if ((clutter_alpha_get_mode (key_animator->alpha) != next_key->mode))
+                clutter_alpha_set_mode (key_animator->alpha, next_key->mode);
             }
           else
-            {
-              next_key = initial_key;
-
-              key_animator->end = 1.0;
-            }
-
-          clutter_interval_set_final_value (key_animator->interval,
-                                            &next_key->value);
-          if ((clutter_alpha_get_mode (key_animator->alpha) != next_key->mode))
-            clutter_alpha_set_mode (key_animator->alpha, next_key->mode);
+            break;
         }
     }
 }
@@ -551,6 +590,7 @@ static gfloat
 list_try_get_rel (GList *list,
                   gint   count)
 {
+  ClutterAnimatorKey *key;
   GList *iter = list;
   GList *best = list;
 
@@ -575,7 +615,14 @@ list_try_get_rel (GList *list,
         }
     }
 
-  return g_value_get_float (&(((ClutterAnimatorKey *)best->data)->value));
+  if (best != NULL && best->data != NULL)
+    {
+      key = best->data;
+
+      return g_value_get_float (&(key->value));
+    }
+
+  return 0;
 }
 
 static void
@@ -613,10 +660,10 @@ animation_animator_new_frame (ClutterTimeline  *timeline,
       /* do not change values if we're not active yet (delay) */
       if (sub_progress >= 0.0 && sub_progress <= 1.0)
         {
-          GValue cvalue = { 0, };
+          GValue tmp_value = { 0, };
           GType int_type;
 
-          g_value_init (&cvalue, G_VALUE_TYPE (&start_key->value));
+          g_value_init (&tmp_value, G_VALUE_TYPE (&start_key->value));
 
           clutter_timeline_advance (animator->priv->slave_timeline,
                                     sub_progress * 10000);
@@ -643,8 +690,8 @@ animation_animator_new_frame (ClutterTimeline  *timeline,
                 {
                   /* interpolated and easing in */
                   clutter_interval_get_initial_value (key_animator->interval,
-                                                      &cvalue);
-                  prev = current = g_value_get_float (&cvalue);
+                                                      &tmp_value);
+                  prev = current = g_value_get_float (&tmp_value);
                 }
 
                next = list_try_get_rel (key_animator->current, 1);
@@ -652,18 +699,18 @@ animation_animator_new_frame (ClutterTimeline  *timeline,
                res = cubic_interpolation (sub_progress, prev, current, next,
                                           nextnext);
 
-               g_value_set_float (&cvalue, res);
+               g_value_set_float (&tmp_value, res);
             }
           else
             clutter_interval_compute_value (key_animator->interval,
                                             sub_progress,
-                                            &cvalue);
+                                            &tmp_value);
 
           g_object_set_property (prop_actor_key->object,
                                  prop_actor_key->property_name,
-                                 &cvalue);
+                                 &tmp_value);
 
-          g_value_unset (&cvalue);
+          g_value_unset (&tmp_value);
         }
     }
 }
@@ -731,20 +778,20 @@ animation_animator_started (ClutterTimeline *timeline,
 
         if (key_animator->ease_in)
           {
-            GValue cvalue = { 0, };
+            GValue tmp_value = { 0, };
             GType int_type;
 
             int_type = clutter_interval_get_value_type (key_animator->interval);
-            g_value_init (&cvalue, int_type);
+            g_value_init (&tmp_value, int_type);
 
             g_object_get_property (initial_key->object,
                                    initial_key->property_name,
-                                   &cvalue);
+                                   &tmp_value);
 
             clutter_interval_set_initial_value (key_animator->interval,
-                                                &cvalue);
+                                                &tmp_value);
 
-            g_value_unset (&cvalue);
+            g_value_unset (&tmp_value);
           }
 
         next = g_list_find_custom (initial->next, key, sort_actor_prop_func);
@@ -767,6 +814,193 @@ animation_animator_started (ClutterTimeline *timeline,
   }
 }
 
+/**
+ * clutter_animator_compute_value:
+ * @animator: a #ClutterAnimator
+ * @object: a #GObject
+ * @property_name: the name of the property on object to check
+ * @progress: a value between 0.0 and 1.0
+ * @value: an initialized value to store the computed result
+ *
+ * Compute the value for a managed property at a given progress.
+ *
+ * If the property is an ease-in property, the current value of the property
+ * on the object will be used as the starting point for computation.
+ *
+ * Return value: %TRUE if the computation yields has a value, otherwise (when
+ *   an error occurs or the progress is before any of the keys) %FALSE is
+ *   returned and the #GValue is left untouched
+ *
+ * Since: 1.2
+ */
+gboolean
+clutter_animator_compute_value (ClutterAnimator *animator,
+                                GObject         *object,
+                                const gchar     *property_name,
+                                gdouble          progress,
+                                GValue          *value)
+{
+  ClutterAnimatorKey   key;
+  ClutterAnimatorKey  *previous;
+  ClutterAnimatorKey  *next = NULL;
+  GParamSpec          *pspec;
+  GList               *initial_l;
+  GList               *previous_l;
+  GList               *next_l;
+  gboolean             ease_in;
+  ClutterInterpolation interpolation;
+ 
+  g_return_val_if_fail (CLUTTER_IS_ANIMATOR (animator), FALSE);
+  g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
+  g_return_val_if_fail (property_name, FALSE);
+  g_return_val_if_fail (value, FALSE);
+  
+  ease_in = clutter_animator_property_get_ease_in (animator, object,
+                                                   property_name);
+  interpolation = clutter_animator_property_get_interpolation (animator,
+                                                   object, property_name);
+
+  property_name = g_intern_string (property_name);
+
+  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (object),
+                                        property_name);
+
+  key.object        = object;
+  key.property_name = property_name;
+
+  initial_l = g_list_find_custom (animator->priv->score, &key,
+                                  sort_actor_prop_func);
+  if (!initial_l)
+    return FALSE;
+
+  /* first find the interval we belong in, that is the first interval
+   * existing between keys
+   */
+
+  for (previous_l = initial_l, next_l = previous_l->next ;
+       previous_l->next ;
+       previous_l = previous_l->next, next_l = previous_l->next)
+    {
+       previous = previous_l->data;
+       if (next_l)
+         {
+           next = next_l->data;
+           if (next->object != object ||
+               next->property_name != property_name)
+             {
+               next_l = NULL;
+               next = NULL;
+             }
+         }
+       else
+         {
+           next = NULL;
+         }
+
+       if (progress < previous->progress)
+         {
+            /* we are before the defined values */
+
+            /* value has not been set */
+            return FALSE;
+         }
+
+       if (!next && previous->progress <= progress)
+         {
+            /* we only had one key for this object/property */
+            /* and we are past it, that is our value */
+            g_value_copy (&previous->value, value);
+            return TRUE;
+         }
+       if (next->progress >= progress)
+         {
+            ClutterInterval *interval;
+            ClutterAlpha    *alpha;
+
+            gdouble sub_progress = (progress - previous->progress)
+                                    / (next->progress - previous->progress);
+            /* this should be our interval */
+            interval = g_object_new (CLUTTER_TYPE_INTERVAL,
+                                     "value-type", pspec->value_type,
+                                     NULL);
+
+            if (ease_in && previous_l == initial_l)
+              {
+                GValue tmp_value = {0, };
+                g_value_init (&tmp_value, pspec->value_type);
+                g_object_get_property (object, property_name, &tmp_value);
+                clutter_interval_set_initial_value (interval, &tmp_value);
+                g_value_unset (&tmp_value);
+              }
+            else
+              {
+                clutter_interval_set_initial_value (interval, &previous->value);
+              }
+            clutter_interval_set_final_value (interval, &next->value);
+
+            alpha = clutter_alpha_new ();
+            clutter_alpha_set_timeline (alpha,
+                                        animator->priv->slave_timeline);
+            clutter_alpha_set_mode (alpha, next->mode);
+
+            clutter_timeline_advance (animator->priv->slave_timeline,
+                                      sub_progress * 10000);
+            sub_progress = clutter_alpha_get_alpha (alpha);
+
+            if (interpolation == CLUTTER_INTERPOLATION_CUBIC &&
+                pspec->value_type == G_TYPE_FLOAT)
+              {
+                gdouble prev, current, nextv, nextnext;
+                gdouble res;
+
+                if ((ease_in == FALSE ||
+                    (ease_in &&
+                     list_find_custom_reverse (previous_l->prev,
+                                               previous_l->data,
+                                               sort_actor_prop_func))))
+                  {
+                    current = g_value_get_float (&previous->value);
+                    prev = list_try_get_rel (previous_l, -1);
+                  }
+                else
+                  {
+                    /* interpolated and easing in */
+                    GValue tmp_value = {0, };
+                    g_value_init (&tmp_value, pspec->value_type);
+                    clutter_interval_get_initial_value (interval,
+                                                        &tmp_value);
+                    prev = current = g_value_get_float (&tmp_value);
+                    g_value_unset (&tmp_value);
+                  }
+
+                 nextv = list_try_get_rel (previous_l, 1);
+                 nextnext = list_try_get_rel (previous_l, 2);
+                 res = cubic_interpolation (sub_progress, prev, current, nextv,
+                                            nextnext);
+                 g_value_set_float (value, res);
+              }
+            else
+              clutter_interval_compute_value (interval,
+                                              sub_progress,
+                                              value);
+
+            g_object_ref_sink (interval);
+            g_object_unref (interval);
+            g_object_ref_sink (alpha);
+            g_object_unref (alpha);
+
+            return TRUE;
+         }
+
+    }
+
+  /* We're at, or past the end, use the last value */
+  g_value_copy (&next->value, value);
+
+  return TRUE;
+}
+
+
 /**
  * clutter_animator_set_timeline:
  * @animator: a #ClutterAnimator
@@ -817,7 +1051,9 @@ clutter_animator_set_timeline (ClutterAnimator *animator,
  *
  * Get the timeline hooked up for driving the #ClutterAnimator
  *
- * Return value: the #ClutterTimeline that drives the animator.
+ * Return value: (transfer none): the #ClutterTimeline that drives the animator
+ *
+ * Since: 1.2
  */
 ClutterTimeline *
 clutter_animator_get_timeline (ClutterAnimator *animator)
@@ -827,16 +1063,18 @@ clutter_animator_get_timeline (ClutterAnimator *animator)
 }
 
 /**
- * clutter_animator_run:
+ * clutter_animator_start:
  * @animator: a #ClutterAnimator
  *
  * Start the ClutterAnimator, this is a thin wrapper that rewinds
  * and starts the animators current timeline.
  *
  * Return value: the #ClutterTimeline that drives the animator.
+ *
+ * Since: 1.2
  */
 ClutterTimeline *
-clutter_animator_run (ClutterAnimator *animator)
+clutter_animator_start (ClutterAnimator *animator)
 {
   ClutterAnimatorPrivate *priv;
 
@@ -953,9 +1191,14 @@ clutter_animator_set (ClutterAnimator *animator,
           break;
         }
 
-      /* FIXME - Depend on GLib 2.24 and use G_VALUE_COLLECT_INIT() */
+#if GLIB_CHECK_VERSION (2, 23, 2)
+      G_VALUE_COLLECT_INIT (&value, G_PARAM_SPEC_VALUE_TYPE (pspec),
+                            args, 0,
+                            &error);
+#else
       g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
       G_VALUE_COLLECT (&value, args, 0, &error);
+#endif /* GLIB_CHECK_VERSION (2, 23, 2) */
 
       if (error)
         {
@@ -1039,15 +1282,13 @@ clutter_animator_set_key (ClutterAnimator *animator,
                           gdouble          progress,
                           const GValue    *value)
 {
-  ClutterAnimatorPrivate *priv;
-  ClutterAnimatorKey     *animator_key;
+  ClutterAnimatorKey *animator_key;
 
   g_return_val_if_fail (CLUTTER_IS_ANIMATOR (animator), NULL);
   g_return_val_if_fail (G_IS_OBJECT (object), NULL);
   g_return_val_if_fail (property_name, NULL);
   g_return_val_if_fail (value, NULL);
 
-  priv = animator->priv;
   property_name = g_intern_string (property_name);
 
   animator_key = clutter_animator_key_new (animator,
@@ -1066,9 +1307,11 @@ clutter_animator_set_key (ClutterAnimator *animator,
 /**
  * clutter_animator_get_keys:
  * @animator: a #ClutterAnimator instance
- * @object: a #GObject to search for or NULL for all
- * @property_name: a specific property name to query for or NULL for all
- * @progress: a specific progress to search for or a negative value for all
+ * @object: (allow-none): a #GObject to search for, or %NULL for all objects
+ * @property_name: (allow-none): a specific property name to query for,
+ *   or %NULL for all properties
+ * @progress: a specific progress to search for, or a negative value for all
+ *   progresses
  *
  * Returns a list of pointers to opaque structures with accessor functions
  * that describe the keys added to an animator.
@@ -1082,7 +1325,7 @@ clutter_animator_set_key (ClutterAnimator *animator,
  */
 GList *
 clutter_animator_get_keys (ClutterAnimator *animator,
-                           GObject         *object,/* or NULL for all */
+                           GObject         *object,
                            const gchar     *property_name,
                            gdouble          progress)
 {
@@ -1090,6 +1333,7 @@ clutter_animator_get_keys (ClutterAnimator *animator,
   GList *k;
 
   g_return_val_if_fail (CLUTTER_IS_ANIMATOR (animator), NULL);
+  g_return_val_if_fail (object == NULL || G_IS_OBJECT (object), NULL);
 
   property_name = g_intern_string (property_name);
 
@@ -1109,10 +1353,13 @@ clutter_animator_get_keys (ClutterAnimator *animator,
 }
 
 /**
- * clutter_animator_remove:
- * @object: a #GObject to search for or NULL for all
- * @property_name: a specific property name to query for or NULL for all
- * @progress: a specific progress to search for or a negative value for all
+ * clutter_animator_remove_key:
+ * @animator: a #ClutterAnimator
+ * @object: (allow-none): a #GObject to search for, or %NULL for all
+ * @property_name: (allow-none): a specific property name to query for,
+ *   or %NULL for all
+ * @progress: a specific progress to search for or a negative value
+ *   for all
  *
  * Removes all keys matching the conditions specificed in the arguments.
  *
@@ -1128,8 +1375,7 @@ clutter_animator_remove_key (ClutterAnimator *animator,
   GList *k;
 
   g_return_if_fail (CLUTTER_IS_ANIMATOR (animator));
-  g_return_if_fail (G_IS_OBJECT (object));
-  g_return_if_fail (property_name != NULL);
+  g_return_if_fail (object == NULL || G_IS_OBJECT (object));
 
   property_name = g_intern_string (property_name);
 
@@ -1172,6 +1418,9 @@ again:
     }
 }
 
+
+
+
 typedef struct _ParseClosure {
   ClutterAnimator *animator;
   ClutterScript *script;
@@ -1288,7 +1537,11 @@ parse_animator_property (JsonArray *array,
       return;
     }
 
-  valid_keys = NULL;
+  if (G_IS_VALUE (clos->value))
+    valid_keys = g_slist_reverse (g_value_get_pointer (clos->value));
+  else
+    g_value_init (clos->value, G_TYPE_POINTER);
+
   for (k = json_array_get_elements (keys);
        k != NULL;
        k = k->next)
@@ -1330,7 +1583,6 @@ parse_animator_property (JsonArray *array,
       valid_keys = g_slist_prepend (valid_keys, animator_key);
     }
 
-  g_value_init (clos->value, G_TYPE_POINTER);
   g_value_set_pointer (clos->value, g_slist_reverse (valid_keys));
 
   clos->result = TRUE;
@@ -1407,6 +1659,10 @@ clutter_animator_set_property (GObject      *gobject,
       clutter_animator_set_duration (self, g_value_get_uint (value));
       break;
 
+    case PROP_TIMELINE:
+      clutter_animator_set_timeline (self, g_value_get_object (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -1427,6 +1683,10 @@ clutter_animator_get_property (GObject    *gobject,
       g_value_set_uint (value, clutter_timeline_get_duration (priv->timeline));
       break;
 
+    case PROP_TIMELINE:
+      g_value_set_object (value, priv->timeline);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -1460,6 +1720,21 @@ clutter_animator_class_init (ClutterAnimatorClass *klass)
                              2000,
                              CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_DURATION, pspec);
+
+  /**
+   * ClutterAnimator:timeline:
+   *
+   * The #ClutterTimeline used by the #ClutterAnimator to drive the
+   * animation
+   *
+   * Since: 1.2
+   */
+  pspec = g_param_spec_object ("timeline",
+                               "Timeline",
+                               "The timeline of the animation",
+                               CLUTTER_TYPE_TIMELINE,
+                               CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_TIMELINE, pspec);
 }
 
 static void
@@ -1561,11 +1836,10 @@ clutter_animator_property_set_ease_in (ClutterAnimator *animator,
 
 
 /**
- * clutter_animator_property_set_interpolation:
+ * clutter_animator_property_get_interpolation:
  * @animator: a #ClutterAnimatorKey
  * @object: a #GObject
  * @property_name: the name of a property on object
- * @interpolation: the #ClutterInterpolation to use
  *
  * Get the interpolation used by animator for a property on a particular
  * object.
@@ -1574,10 +1848,9 @@ clutter_animator_property_set_ease_in (ClutterAnimator *animator,
  * Since: 1.2
  */
 ClutterInterpolation
-clutter_animator_property_get_interpolation (ClutterAnimator      *animator,
-                                             GObject              *object,
-                                             const gchar          *property_name,
-                                             ClutterInterpolation  interpolation)
+clutter_animator_property_get_interpolation (ClutterAnimator *animator,
+                                             GObject         *object,
+                                             const gchar     *property_name)
 {
   GList              *initial;
   ClutterAnimatorKey  key, *initial_key;
@@ -1610,17 +1883,18 @@ clutter_animator_property_get_interpolation (ClutterAnimator      *animator,
  * @property_name: the name of a property on object
  * @interpolation: the #ClutterInterpolation to use
  *
- * Set the interpolation method to use, CLUTTER_INTERPOLATION_LINEAR causes the
- * values to linearly change between the values, CLUTTER_INTERPOLATION_CUBIC
- * causes the values to smoothly change between the values.
+ * Set the interpolation method to use, %CLUTTER_INTERPOLATION_LINEAR causes
+ * the values to linearly change between the values, and
+ * %CLUTTER_INTERPOLATION_CUBIC causes the values to smoothly change between
+ * the values.
  *
  * Since: 1.2
  */
 void
-clutter_animator_property_set_interpolation (ClutterAnimator      *animator,
-                                             GObject              *object,
-                                             const gchar          *property_name,
-                                             ClutterInterpolation  interpolation)
+clutter_animator_property_set_interpolation (ClutterAnimator     *animator,
+                                             GObject             *object,
+                                             const gchar         *property_name,
+                                             ClutterInterpolation interpolation)
 {
   GList              *initial;
   ClutterAnimatorKey  key, *initial_key;
