@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /**
@@ -360,6 +360,7 @@ clutter_group_dispose (GObject *object)
 
   if (priv->layout)
     {
+      clutter_layout_manager_set_container (priv->layout, NULL);
       g_object_unref (priv->layout);
       priv->layout = NULL;
     }
@@ -416,11 +417,8 @@ clutter_group_init (ClutterGroup *self)
   self->priv->layout = clutter_fixed_layout_new ();
   g_object_ref_sink (self->priv->layout);
 
-  /* signal Clutter that we don't impose any layout on
-   * our children, so we can shave off some relayout
-   * operations
-   */
-  CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_NO_LAYOUT);
+  clutter_layout_manager_set_container (self->priv->layout,
+                                        CLUTTER_CONTAINER (self));
 }
 
 /**
