@@ -18,22 +18,18 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
-#ifndef _HAVE_CLUTTER_PRIVATE_H
-#define _HAVE_CLUTTER_PRIVATE_H
+#ifndef __CLUTTER_PRIVATE_H__
+#define __CLUTTER_PRIVATE_H__
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include <math.h>
 
 #include <glib.h>
@@ -127,20 +123,13 @@ struct _ClutterStageManager
   GSList *stages;
 };
 
-struct _ClutterDeviceManager
-{
-  GObject parent_instance;
-
-  GSList *devices;
-};
-
 struct _ClutterMainContext
 {
-  ClutterBackend  *backend;            /* holds a pointer to the windowing 
+  ClutterBackend  *backend;            /* holds a pointer to the windowing
                                           system backend */
   GQueue          *events_queue;       /* the main event queue */
 
-  guint            is_initialized : 1;  
+  guint            is_initialized : 1;
   guint            motion_events_per_actor : 1;/* set for enter/leave events */
   guint            defer_display_setup : 1;
   guint            options_parsed : 1;
@@ -151,8 +140,8 @@ struct _ClutterMainContext
 
   gint             num_reactives;      /* Num of reactive actors */
 
-  ClutterIDPool   *id_pool;            /* mapping between reused integer ids 
-                                        * and actors 
+  ClutterIDPool   *id_pool;            /* mapping between reused integer ids
+                                        * and actors
                                         */
   guint            frame_rate;         /* Default FPS */
 
@@ -160,8 +149,8 @@ struct _ClutterMainContext
                                         * (or NULL if there is no pointer grab
                                         */
   ClutterActor    *keyboard_grab_actor; /* The actor having the pointer grab
-                                         * (or NULL if there is no pointer 
-                                         *  grab) 
+                                         * (or NULL if there is no pointer
+                                         *  grab)
                                          */
   GSList          *shaders;            /* stack of overridden shaders */
 
@@ -223,10 +212,12 @@ void          _clutter_input_device_set_actor  (ClutterInputDevice  *device,
 ClutterActor *_clutter_input_device_update     (ClutterInputDevice  *device);
 
 /* stage manager */
-void _clutter_stage_manager_add_stage    (ClutterStageManager *stage_manager,
-                                          ClutterStage        *stage);
-void _clutter_stage_manager_remove_stage (ClutterStageManager *stage_manager,
-                                          ClutterStage        *stage);
+void _clutter_stage_manager_add_stage         (ClutterStageManager *stage_manager,
+                                               ClutterStage        *stage);
+void _clutter_stage_manager_remove_stage      (ClutterStageManager *stage_manager,
+                                               ClutterStage        *stage);
+void _clutter_stage_manager_set_default_stage (ClutterStageManager *stage_manager,
+                                               ClutterStage        *stage);
 
 /* stage */
 void                _clutter_stage_set_window           (ClutterStage       *stage,
@@ -238,13 +229,17 @@ void                _clutter_stage_maybe_relayout       (ClutterActor       *sta
 gboolean            _clutter_stage_needs_update         (ClutterStage       *stage);
 gboolean            _clutter_stage_do_update            (ClutterStage       *stage);
 
-void     _clutter_stage_queue_event           (ClutterStage *stage,
-					       ClutterEvent *event);
-gboolean _clutter_stage_has_queued_events     (ClutterStage *stage);
-void     _clutter_stage_process_queued_events (ClutterStage *stage);
-void     _clutter_stage_update_input_devices  (ClutterStage *stage);
 
-int      _clutter_stage_get_pending_swaps     (ClutterStage *stage);
+void     _clutter_stage_queue_event            (ClutterStage *stage,
+					        ClutterEvent *event);
+gboolean _clutter_stage_has_queued_events      (ClutterStage *stage);
+void     _clutter_stage_process_queued_events  (ClutterStage *stage);
+void     _clutter_stage_update_input_devices   (ClutterStage *stage);
+
+int      _clutter_stage_get_pending_swaps      (ClutterStage *stage);
+
+gboolean _clutter_stage_has_full_redraw_queued (ClutterStage *stage);
+
 
 /* vfuncs implemented by backend */
 GType         _clutter_backend_impl_get_type  (void);
@@ -321,6 +316,17 @@ void _clutter_actor_set_enable_paint_unmapped (ClutterActor *self,
 void _clutter_actor_set_has_pointer (ClutterActor *self,
                                      gboolean      has_pointer);
 
+void _clutter_actor_transform_and_project_box (ClutterActor          *self,
+					       const ClutterActorBox *box,
+					       ClutterVertex          verts[]);
+
+void _clutter_actor_queue_redraw_with_clip (ClutterActor          *self,
+                                            ClutterRedrawFlags     flags,
+                                            ClutterActorBox       *clip);
+const ClutterActorBox *_clutter_actor_get_queue_redraw_clip (ClutterActor *self);
+void _clutter_actor_set_queue_redraw_clip (ClutterActor *self,
+                                           const ClutterActorBox *clip);
+
 void _clutter_run_repaint_functions (void);
 
 gint32 _clutter_backend_get_units_serial (ClutterBackend *backend);
