@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2009 Intel Corporation.
+ * Copyright (C) 2009,2010 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * Authors:
  *  Neil Roberts   <neil@linux.intel.com>
@@ -39,6 +39,7 @@
 #include "cogl-handle.h"
 #include "cogl-texture-driver.h"
 #include "cogl-atlas.h"
+#include "cogl-journal-private.h"
 
 #include <stdlib.h>
 
@@ -92,8 +93,8 @@ typedef struct _CoglAtlasTextureBlitData
      complete texture data in */
   unsigned char *image_data;
   CoglPixelFormat format;
-  gint bpp;
-  guint src_height, src_width;
+  int bpp;
+  unsigned int src_height, src_width;
 
   GLenum dst_gl_target;
 } CoglAtlasTextureBlitData;
@@ -169,12 +170,12 @@ _cogl_atlas_texture_blit_begin (CoglAtlasTextureBlitData *data,
 
 static void
 _cogl_atlas_texture_blit (CoglAtlasTextureBlitData *data,
-                          guint src_x,
-                          guint src_y,
-                          guint dst_x,
-                          guint dst_y,
-                          guint width,
-                          guint height)
+                          unsigned int src_x,
+                          unsigned int src_y,
+                          unsigned int dst_x,
+                          unsigned int dst_y,
+                          unsigned int width,
+                          unsigned int height)
 {
   /* If we have an FBO then we can do a fast blit */
   if (data->fbo)
@@ -269,7 +270,7 @@ _cogl_atlas_texture_free (CoglAtlasTexture *atlas_tex)
   cogl_handle_unref (atlas_tex->sub_texture);
 }
 
-static gint
+static int
 _cogl_atlas_texture_get_max_waste (CoglTexture *tex)
 {
   CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
@@ -354,6 +355,13 @@ _cogl_atlas_texture_migrate_out_of_atlas (CoglAtlasTexture *atlas_tex)
 
       COGL_NOTE (ATLAS, "Migrating texture out of the atlas");
 
+      /* Although this texture is being moved out of the atlas, we
+         don't need to flush the journal here because the contents of
+         the old image will still be left in the same position in the
+         atlas. The only way this would get overwritten would be if a
+         new texture was added, but in that case we flush the journal
+         anyway */
+
       cogl_handle_unref (atlas_tex->sub_texture);
 
       /* Create a new texture at the right size, not including the
@@ -506,7 +514,7 @@ _cogl_atlas_texture_set_region (CoglTexture    *tex,
      pixels to the border */
   if (atlas_tex->in_atlas)
     {
-      gint             bpp;
+      int              bpp;
       CoglBitmap       source_bmp;
       CoglBitmap       tmp_bmp;
       gboolean         tmp_bmp_owner = FALSE;
@@ -524,7 +532,7 @@ _cogl_atlas_texture_set_region (CoglTexture    *tex,
       source_bmp.width = width;
       source_bmp.height = height;
       source_bmp.format = format;
-      source_bmp.data = (guchar*) data;
+      source_bmp.data = (guint8 *)data;
 
       /* Rowstride from width if none specified */
       bpp = _cogl_get_format_bpp (format);
@@ -604,7 +612,7 @@ _cogl_atlas_texture_get_gl_format (CoglTexture *tex)
   return _cogl_texture_get_gl_format (atlas_tex->sub_texture);
 }
 
-static gint
+static int
 _cogl_atlas_texture_get_width (CoglTexture *tex)
 {
   CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
@@ -613,7 +621,7 @@ _cogl_atlas_texture_get_width (CoglTexture *tex)
   return cogl_texture_get_width (atlas_tex->sub_texture);
 }
 
-static gint
+static int
 _cogl_atlas_texture_get_height (CoglTexture *tex)
 {
   CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
@@ -644,13 +652,13 @@ typedef struct _CoglAtlasTextureRepositionData
 } CoglAtlasTextureRepositionData;
 
 static void
-_cogl_atlas_texture_migrate (guint                           n_textures,
+_cogl_atlas_texture_migrate (unsigned int                    n_textures,
                              CoglAtlasTextureRepositionData *textures,
                              CoglHandle                      old_texture,
                              CoglHandle                      new_texture,
                              CoglAtlasTexture               *skip_texture)
 {
-  guint i;
+  unsigned int i;
   CoglAtlasTextureBlitData blit_data;
 
   _cogl_atlas_texture_blit_begin (&blit_data, new_texture, old_texture);
@@ -686,7 +694,7 @@ typedef struct _CoglAtlasTextureGetRectanglesData
 {
   CoglAtlasTextureRepositionData *textures;
   /* Number of textures found so far */
-  guint n_textures;
+  unsigned int n_textures;
 } CoglAtlasTextureGetRectanglesData;
 
 static void
@@ -700,8 +708,8 @@ _cogl_atlas_texture_get_rectangles_cb (const CoglAtlasRectangle *rectangle,
 }
 
 static void
-_cogl_atlas_texture_get_next_size (guint *atlas_width,
-                                   guint *atlas_height)
+_cogl_atlas_texture_get_next_size (unsigned int *atlas_width,
+                                   unsigned int *atlas_height)
 {
   /* Double the size of the texture by increasing whichever dimension
      is smaller */
@@ -712,9 +720,9 @@ _cogl_atlas_texture_get_next_size (guint *atlas_width,
 }
 
 static CoglAtlas *
-_cogl_atlas_texture_create_atlas (guint                           atlas_width,
-                                  guint                           atlas_height,
-                                  guint                           n_textures,
+_cogl_atlas_texture_create_atlas (unsigned int                    atlas_width,
+                                  unsigned int                    atlas_height,
+                                  unsigned int                    n_textures,
                                   CoglAtlasTextureRepositionData *textures)
 {
   GLint max_texture_size = 1024;
@@ -731,7 +739,7 @@ _cogl_atlas_texture_create_atlas (guint                           atlas_width,
   while (atlas_width < max_texture_size && atlas_height < max_texture_size)
     {
       CoglAtlas *new_atlas = _cogl_atlas_new (atlas_width, atlas_height, NULL);
-      guint i;
+      unsigned int i;
 
       /* Add all of the textures and keep track of the new position */
       for (i = 0; i < n_textures; i++)
@@ -763,7 +771,7 @@ _cogl_atlas_texture_compare_size_cb (const void *a,
 {
   const CoglAtlasTextureRepositionData *ta = a;
   const CoglAtlasTextureRepositionData *tb = b;
-  guint a_size, b_size;
+  unsigned int a_size, b_size;
 
   a_size = ta->texture->rectangle.width * ta->texture->rectangle.height;
   b_size = tb->texture->rectangle.width * tb->texture->rectangle.height;
@@ -773,13 +781,13 @@ _cogl_atlas_texture_compare_size_cb (const void *a,
 
 static gboolean
 _cogl_atlas_texture_reserve_space (CoglAtlasTexture    *new_sub_tex,
-                                   guint                width,
-                                   guint                height)
+                                   unsigned int         width,
+                                   unsigned int         height)
 {
   CoglAtlasTextureGetRectanglesData data;
   CoglAtlas *new_atlas;
   CoglHandle new_tex;
-  guint atlas_width, atlas_height;
+  unsigned int atlas_width, atlas_height;
   gboolean ret;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
@@ -853,15 +861,20 @@ _cogl_atlas_texture_reserve_space (CoglAtlasTexture    *new_sub_tex,
       COGL_NOTE (ATLAS, "Could not fit texture in the atlas");
       ret = FALSE;
     }
+  /* We need to migrate the existing textures into a new texture */
+  else if ((new_tex =
+            _cogl_texture_2d_new_with_size (_cogl_atlas_get_width (new_atlas),
+                                            _cogl_atlas_get_height (new_atlas),
+                                            COGL_TEXTURE_NONE,
+                                            COGL_PIXEL_FORMAT_RGBA_8888)) ==
+           COGL_INVALID_HANDLE)
+    {
+      COGL_NOTE (ATLAS, "Could not create a CoglTexture2D");
+      _cogl_atlas_free (new_atlas);
+      ret = FALSE;
+    }
   else
     {
-      /* We need to migrate the existing textures into a new texture */
-      new_tex =
-        _cogl_texture_2d_new_with_size (_cogl_atlas_get_width (new_atlas),
-                                        _cogl_atlas_get_height (new_atlas),
-                                        COGL_TEXTURE_NONE,
-                                        COGL_PIXEL_FORMAT_RGBA_8888);
-
       COGL_NOTE (ATLAS,
                  "Atlas %s with size %ix%i",
                  ctx->atlas == NULL ||
@@ -942,9 +955,11 @@ _cogl_atlas_texture_new_from_bitmap (CoglHandle       bmp_handle,
   if (bmp->width < 1 || bmp->height < 1)
     return COGL_INVALID_HANDLE;
 
-  /* If we can't read back texture data then it will be too slow to
-     migrate textures and we shouldn't use the atlas */
-  if (!cogl_features_available (COGL_FEATURE_TEXTURE_READ_PIXELS))
+  /* If we can't use FBOs or we can't read back texture data then it
+     will be too slow to migrate textures and we shouldn't use the
+     atlas */
+  if (!cogl_features_available (COGL_FEATURE_TEXTURE_READ_PIXELS) ||
+      !cogl_features_available (COGL_FEATURE_OFFSCREEN))
     return COGL_INVALID_HANDLE;
 
   COGL_NOTE (ATLAS, "Adding texture of size %ix%i", bmp->width, bmp->height);
@@ -962,6 +977,22 @@ _cogl_atlas_texture_new_from_bitmap (CoglHandle       bmp_handle,
       return COGL_INVALID_HANDLE;
     }
 
+  /* We need to flush the journal when adding a new atlas texture for
+     two reasons:
+
+     1) If the atlas gets reorganized then any existing textures may
+        be moved so the coordinates in journal will be wrong.
+
+     2) We don't want to flush before migrating a texture out of the
+        atlas because we will potentially already be in the middle of
+        a flush to cause the migration. Therefore we rely on the old
+        contents of the texture still remaining in the atlas so that
+        any existing journal entries will still be valid. However once
+        the texture is migrated out the space would be free to be
+        reclaimed by this new texture. That won't matter if we flush
+        now */
+  _cogl_journal_flush ();
+
   /* We need to allocate the texture now because we need the pointer
      to set as the data for the rectangle in the atlas */
   atlas_tex = g_new (CoglAtlasTexture, 1);
