@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,6 +37,7 @@
 #include "cogl-internal.h"
 #include "cogl-framebuffer-private.h"
 #include "cogl-journal-private.h"
+#include "cogl-util.h"
 
 void _cogl_add_path_to_stencil_buffer (floatVec2    nodes_min,
                                        floatVec2    nodes_max,
@@ -483,7 +484,10 @@ try_pushing_rect_as_window_rect (float x_1,
     SWAP (y_1, y_2);
 #undef SWAP
 
-  cogl_clip_push_window_rectangle (x_1, y_1, x_2 - x_1, y_2 - y_1);
+  cogl_clip_push_window_rectangle (COGL_UTIL_NEARBYINT (x_1),
+                                   COGL_UTIL_NEARBYINT (y_1),
+                                   COGL_UTIL_NEARBYINT (x_2 - x_1),
+                                   COGL_UTIL_NEARBYINT (y_2 - y_1));
   return TRUE;
 }
 
