@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2007,2008,2009 Intel Corporation.
+ * Copyright (C) 2007,2008,2009,2010 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,12 +32,17 @@
 #include "cogl-texture-private.h"
 #include "cogl-material-private.h"
 #include "cogl-vertex-buffer-private.h"
+#include "cogl-framebuffer-private.h"
 
 #include <string.h>
 #include <math.h>
 
 #define _COGL_MAX_BEZ_RECURSE_DEPTH 16
 
+#ifdef HAVE_COGL_GL
+#define glClientActiveTexture ctx->drv.pf_glClientActiveTexture
+#endif
+
 
 typedef struct _TextureSlicedQuadState
 {
@@ -56,24 +61,18 @@ typedef struct _TextureSlicedQuadState
 
 typedef struct _TextureSlicedPolygonState
 {
-  CoglTextureVertex *vertices;
+  const CoglTextureVertex *vertices;
   int n_vertices;
   int stride;
 } TextureSlicedPolygonState;
 
-/* these are defined in the particular backend */
-void _cogl_path_add_node    (gboolean new_sub_path,
-			     float x,
-                             float y);
-void _cogl_path_fill_nodes    ();
-void _cogl_path_stroke_nodes  ();
 
 static void
 log_quad_sub_textures_cb (CoglHandle texture_handle,
                           GLuint gl_handle,
                           GLenum gl_target,
-                          float *subtexture_coords,
-                          float *virtual_coords,
+                          const float *subtexture_coords,
+                          const float *virtual_coords,
                           void *user_data)
 {
   TextureSlicedQuadState *state = user_data;
@@ -113,10 +112,7 @@ log_quad_sub_textures_cb (CoglHandle texture_handle,
 
   /* FIXME: when the wrap mode becomes part of the material we need to
    * be able to override the wrap mode when logging a quad. */
-  _cogl_journal_log_quad (quad_coords[0],
-                          quad_coords[1],
-                          quad_coords[2],
-                          quad_coords[3],
+  _cogl_journal_log_quad (quad_coords,
                           state->material,
                           1, /* one layer */
                           0, /* don't need to use fallbacks */
@@ -142,10 +138,7 @@ log_quad_sub_textures_cb (CoglHandle texture_handle,
 static void
 _cogl_texture_quad_multiple_primitives (CoglHandle   tex_handle,
                                         CoglHandle   material,
-                                        float        x_1,
-                                        float        y_1,
-                                        float        x_2,
-                                        float        y_2,
+                                        const float *position,
                                         float        tx_1,
                                         float        ty_1,
                                         float        tx_2,
@@ -176,15 +169,20 @@ _cogl_texture_quad_multiple_primitives (CoglHandle   tex_handle,
    * inversions when we emit the final geometry.
    */
 
+#define X0 0
+#define Y0 1
+#define X1 2
+#define Y1 3
+
   tex_virtual_flipped_x = (tx_1 > tx_2) ? TRUE : FALSE;
   tex_virtual_flipped_y = (ty_1 > ty_2) ? TRUE : FALSE;
   state.tex_virtual_origin_x = tex_virtual_flipped_x ? tx_2 : tx_1;
   state.tex_virtual_origin_y = tex_virtual_flipped_y ? ty_2 : ty_1;
 
-  quad_flipped_x = (x_1 > x_2) ? TRUE : FALSE;
-  quad_flipped_y = (y_1 > y_2) ? TRUE : FALSE;
-  state.quad_origin_x = quad_flipped_x ? x_2 : x_1;
-  state.quad_origin_y = quad_flipped_y ? y_2 : y_1;
+  quad_flipped_x = (position[X0] > position[X1]) ? TRUE : FALSE;
+  quad_flipped_y = (position[Y0] > position[Y1]) ? TRUE : FALSE;
+  state.quad_origin_x = quad_flipped_x ? position[X1] : position[X0];
+  state.quad_origin_y = quad_flipped_y ? position[Y1] : position[Y0];
 
   /* flatten the two forms of coordinate inversion into one... */
   state.flipped_x = tex_virtual_flipped_x ^ quad_flipped_x;
@@ -193,8 +191,13 @@ _cogl_texture_quad_multiple_primitives (CoglHandle   tex_handle,
   /* We use the _len_AXIS naming here instead of _width and _height because
    * log_quad_slice_cb uses a macro with symbol concatenation to handle both
    * axis, so this is more convenient... */
-  state.quad_len_x = fabs (x_2 - x_1);
-  state.quad_len_y = fabs (y_2 - y_1);
+  state.quad_len_x = fabs (position[X1] - position[X0]);
+  state.quad_len_y = fabs (position[Y1] - position[Y0]);
+
+#undef X0
+#undef Y0
+#undef X1
+#undef Y1
 
   state.v_to_q_scale_x = fabs (state.quad_len_x / (tx_2 - tx_1));
   state.v_to_q_scale_y = fabs (state.quad_len_y / (ty_2 - ty_1));
@@ -222,10 +225,7 @@ _cogl_texture_quad_multiple_primitives (CoglHandle   tex_handle,
  *   require repeating.
  */
 static gboolean
-_cogl_multitexture_quad_single_primitive (float        x_1,
-                                          float        y_1,
-                                          float        x_2,
-                                          float        y_2,
+_cogl_multitexture_quad_single_primitive (const float *position,
                                           CoglHandle   material,
                                           guint32      fallback_layers,
                                           const float *user_tex_coords,
@@ -250,6 +250,9 @@ _cogl_multitexture_quad_single_primitive (float        x_1,
       const float       *in_tex_coords;
       float             *out_tex_coords;
       float              default_tex_coords[4] = {0.0, 0.0, 1.0, 1.0};
+      gboolean           need_repeat = FALSE;
+      int               coord_num;
+      GLenum             wrap_mode;
 
       tex_handle = cogl_material_layer_get_texture (layer);
 
@@ -258,27 +261,41 @@ _cogl_multitexture_quad_single_primitive (float        x_1,
       if (tex_handle == COGL_INVALID_HANDLE)
         continue;
 
-      in_tex_coords = &user_tex_coords[i * 4];
+      /* If the user didn't supply texture coordinates for this layer
+         then use the default coords */
+      if (i >= user_tex_coords_len / 4)
+        in_tex_coords = default_tex_coords;
+      else
+        in_tex_coords = &user_tex_coords[i * 4];
+
       out_tex_coords = &final_tex_coords[i * 4];
 
+      memcpy (out_tex_coords, in_tex_coords, sizeof (GLfloat) * 4);
+
+      /* Convert the texture coordinates to GL. We also work out
+         whether any of the texture coordinates are outside the range
+         [0.0,1.0]. We need to do this after calling
+         transform_coords_to_gl in case the texture backend is munging
+         the coordinates (such as in the sub texture backend). This
+         should be safe to call because we know that the texture only
+         has one slice. */
+      if (!_cogl_texture_transform_quad_coords_to_gl (tex_handle,
+                                                      out_tex_coords))
+        /* If the backend can't support these coordinates then bail out */
+        return FALSE;
+      for (coord_num = 0; coord_num < 4; coord_num++)
+        if (out_tex_coords[coord_num] < 0.0f ||
+            out_tex_coords[coord_num] > 1.0f)
+          need_repeat = TRUE;
 
       /* If the texture has waste or we are using GL_TEXTURE_RECT we
-       * can't handle texture repeating so we check that the texture
-       * coords lie in the range [0,1].
-       *
-       * NB: We already know that the texture isn't sliced so we can assume
-       * that the default coords (0,0) and (1,1) would only reference a single
-       * GL texture.
+       * can't handle texture repeating so we can't use the layer if
+       * repeating is required.
        *
        * NB: We already know that no texture matrix is being used if the
        * texture doesn't support hardware repeat.
        */
-      if (!_cogl_texture_can_hardware_repeat (tex_handle)
-           && i < user_tex_coords_len / 4
-           && (in_tex_coords[0] < 0 || in_tex_coords[0] > 1.0
-               || in_tex_coords[1] < 0 || in_tex_coords[1] > 1.0
-               || in_tex_coords[2] < 0 || in_tex_coords[2] > 1.0
-               || in_tex_coords[3] < 0 || in_tex_coords[3] > 1.0))
+      if (!_cogl_texture_can_hardware_repeat (tex_handle) && need_repeat)
         {
           if (i == 0)
             {
@@ -316,51 +333,18 @@ _cogl_multitexture_quad_single_primitive (float        x_1,
             }
         }
 
-
-      /*
-       * Setup the texture unit...
-       */
-
-      /* NB: The user might not have supplied texture coordinates for all
-       * layers... */
-      if (i < (user_tex_coords_len / 4))
-        {
-          GLenum wrap_mode;
-
-          /* If the texture coords are all in the range [0,1] then we want to
-             clamp the coords to the edge otherwise it can pull in edge pixels
-             from the wrong side when scaled */
-          if (in_tex_coords[0] >= 0 && in_tex_coords[0] <= 1.0
-              && in_tex_coords[1] >= 0 && in_tex_coords[1] <= 1.0
-              && in_tex_coords[2] >= 0 && in_tex_coords[2] <= 1.0
-              && in_tex_coords[3] >= 0 && in_tex_coords[3] <= 1.0)
-            wrap_mode = GL_CLAMP_TO_EDGE;
-          else
-            wrap_mode = GL_REPEAT;
-
-          memcpy (out_tex_coords, in_tex_coords, sizeof (GLfloat) * 4);
-
-          _cogl_texture_set_wrap_mode_parameter (tex_handle, wrap_mode);
-        }
+      /* If we're not repeating then we want to clamp the coords
+         to the edge otherwise it can pull in edge pixels from the
+         wrong side when scaled */
+      if (need_repeat)
+        wrap_mode = GL_REPEAT;
       else
-        {
-          memcpy (out_tex_coords, default_tex_coords, sizeof (GLfloat) * 4);
+        wrap_mode = GL_CLAMP_TO_EDGE;
 
-          _cogl_texture_set_wrap_mode_parameter (tex_handle, GL_CLAMP_TO_EDGE);
-        }
-
-      _cogl_texture_transform_coords_to_gl (tex_handle,
-                                            &out_tex_coords[0],
-                                            &out_tex_coords[1]);
-      _cogl_texture_transform_coords_to_gl (tex_handle,
-                                            &out_tex_coords[2],
-                                            &out_tex_coords[3]);
+      _cogl_texture_set_wrap_mode_parameter (tex_handle, wrap_mode);
     }
 
-  _cogl_journal_log_quad (x_1,
-                          y_1,
-                          x_2,
-                          y_2,
+  _cogl_journal_log_quad (position,
                           material,
                           n_layers,
                           fallback_layers,
@@ -373,18 +357,15 @@ _cogl_multitexture_quad_single_primitive (float        x_1,
 
 struct _CoglMutiTexturedRect
 {
-  float        x_1;
-  float        y_1;
-  float        x_2;
-  float        y_2;
-  const float *tex_coords;
-  gint             tex_coords_len;
+  const float *position; /* x0,y0,x1,y1 */
+  const float *tex_coords; /* (tx0,ty0,tx1,ty1)(tx0,ty0,tx1,ty1)(... */
+  int          tex_coords_len; /* number of floats in tex_coords? */
 };
 
 static void
 _cogl_rectangles_with_multitexture_coords (
                                         struct _CoglMutiTexturedRect *rects,
-                                        gint                          n_rects)
+                                        int                           n_rects)
 {
   CoglHandle	 material;
   const GList	*layers;
@@ -396,8 +377,6 @@ _cogl_rectangles_with_multitexture_coords (
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  cogl_clip_ensure ();
-
   material = ctx->source_material;
 
   layers = cogl_material_get_layers (material);
@@ -411,12 +390,18 @@ _cogl_rectangles_with_multitexture_coords (
     {
       CoglHandle     layer = tmp->data;
       CoglHandle     tex_handle;
-      gulong         flags;
+      unsigned long  flags;
 
       if (cogl_material_layer_get_type (layer)
 	  != COGL_MATERIAL_LAYER_TYPE_TEXTURE)
 	continue;
 
+      /* We need to ensure the mipmaps are ready before deciding
+         anything else about the texture because it could become
+         something completely different if it needs to be migrated out
+         of the atlas */
+      _cogl_material_layer_ensure_mipmaps (layer);
+
       tex_handle = cogl_material_layer_get_texture (layer);
 
       /* COGL_INVALID_HANDLE textures are handled by
@@ -505,10 +490,7 @@ _cogl_rectangles_with_multitexture_coords (
       if (!all_use_sliced_quad_fallback)
         {
           gboolean success =
-            _cogl_multitexture_quad_single_primitive (rects[i].x_1,
-                                                      rects[i].y_1,
-                                                      rects[i].x_2,
-                                                      rects[i].y_2,
+            _cogl_multitexture_quad_single_primitive (rects[i].position,
                                                       material,
                                                       fallback_layers,
                                                       rects[i].tex_coords,
@@ -535,8 +517,7 @@ _cogl_rectangles_with_multitexture_coords (
 
       _cogl_texture_quad_multiple_primitives (tex_handle,
                                               material,
-                                              rects[i].x_1, rects[i].y_1,
-                                              rects[i].x_2, rects[i].y_2,
+                                              rects[i].position,
                                               tex_coords[0],
                                               tex_coords[1],
                                               tex_coords[2],
@@ -552,19 +533,21 @@ _cogl_rectangles_with_multitexture_coords (
 
 void
 cogl_rectangles (const float *verts,
-                 guint        n_rects)
+                 unsigned int n_rects)
 {
   struct _CoglMutiTexturedRect *rects;
   int i;
 
+  /* XXX: All the cogl_rectangle* APIs normalize their input into an array of
+   * _CoglMutiTexturedRect rectangles and pass these on to our work horse;
+   * _cogl_rectangles_with_multitexture_coords.
+   */
+
   rects = g_alloca (n_rects * sizeof (struct _CoglMutiTexturedRect));
 
   for (i = 0; i < n_rects; i++)
     {
-      rects[i].x_1 = verts[i * 4];
-      rects[i].y_1 = verts[i * 4 + 1];
-      rects[i].x_2 = verts[i * 4 + 2];
-      rects[i].y_2 = verts[i * 4 + 3];
+      rects[i].position = &verts[i * 4];
       rects[i].tex_coords = NULL;
       rects[i].tex_coords_len = 0;
     }
@@ -574,22 +557,21 @@ cogl_rectangles (const float *verts,
 
 void
 cogl_rectangles_with_texture_coords (const float *verts,
-                                     guint        n_rects)
+                                     unsigned int n_rects)
 {
   struct _CoglMutiTexturedRect *rects;
   int i;
 
+  /* XXX: All the cogl_rectangle* APIs normalize their input into an array of
+   * _CoglMutiTexturedRect rectangles and pass these on to our work horse;
+   * _cogl_rectangles_with_multitexture_coords.
+   */
+
   rects = g_alloca (n_rects * sizeof (struct _CoglMutiTexturedRect));
 
   for (i = 0; i < n_rects; i++)
     {
-      rects[i].x_1 = verts[i * 8];
-      rects[i].y_1 = verts[i * 8 + 1];
-      rects[i].x_2 = verts[i * 8 + 2];
-      rects[i].y_2 = verts[i * 8 + 3];
-      /* FIXME: rect should be defined to have a const float *geom;
-       * instead, to avoid this copy
-       * rect[i].geom = &verts[n_rects * 8]; */
+      rects[i].position = &verts[i * 8];
       rects[i].tex_coords = &verts[i * 8 + 4];
       rects[i].tex_coords_len = 4;
     }
@@ -607,18 +589,20 @@ cogl_rectangle_with_texture_coords (float x_1,
 			            float tx_2,
 			            float ty_2)
 {
-  float verts[8];
+  const float position[4] = {x_1, y_1, x_2, y_2};
+  const float tex_coords[4] = {tx_1, ty_1, tx_2, ty_2};
+  struct _CoglMutiTexturedRect rect;
 
-  verts[0] = x_1;
-  verts[1] = y_1;
-  verts[2] = x_2;
-  verts[3] = y_2;
-  verts[4] = tx_1;
-  verts[5] = ty_1;
-  verts[6] = tx_2;
-  verts[7] = ty_2;
+  /* XXX: All the cogl_rectangle* APIs normalize their input into an array of
+   * _CoglMutiTexturedRect rectangles and pass these on to our work horse;
+   * _cogl_rectangles_with_multitexture_coords.
+   */
 
-  cogl_rectangles_with_texture_coords (verts, 1);
+  rect.position = position;
+  rect.tex_coords = tex_coords;
+  rect.tex_coords_len = 4;
+
+  _cogl_rectangles_with_multitexture_coords (&rect, 1);
 }
 
 void
@@ -627,14 +611,17 @@ cogl_rectangle_with_multitexture_coords (float        x_1,
 			                 float        x_2,
 			                 float        y_2,
 			                 const float *user_tex_coords,
-                                         gint         user_tex_coords_len)
+                                         int          user_tex_coords_len)
 {
+  const float position[4] = {x_1, y_1, x_2, y_2};
   struct _CoglMutiTexturedRect rect;
 
-  rect.x_1 = x_1;
-  rect.y_1 = y_1;
-  rect.x_2 = x_2;
-  rect.y_2 = y_2;
+  /* XXX: All the cogl_rectangle* APIs normalize their input into an array of
+   * _CoglMutiTexturedRect rectangles and pass these on to our work horse;
+   * _cogl_rectangles_with_multitexture_coords.
+   */
+
+  rect.position = position;
   rect.tex_coords = user_tex_coords;
   rect.tex_coords_len = user_tex_coords_len;
 
@@ -647,18 +634,28 @@ cogl_rectangle (float x_1,
                 float x_2,
                 float y_2)
 {
-  cogl_rectangle_with_multitexture_coords (x_1, y_1,
-                                           x_2, y_2,
-                                           NULL, 0);
+  const float position[4] = {x_1, y_1, x_2, y_2};
+  struct _CoglMutiTexturedRect rect;
+
+  /* XXX: All the cogl_rectangle* APIs normalize their input into an array of
+   * _CoglMutiTexturedRect rectangles and pass these on to our work horse;
+   * _cogl_rectangles_with_multitexture_coords.
+   */
+
+  rect.position = position;
+  rect.tex_coords = NULL;
+  rect.tex_coords_len = 0;
+
+  _cogl_rectangles_with_multitexture_coords (&rect, 1);
 }
 
 void
 draw_polygon_sub_texture_cb (CoglHandle tex_handle,
-                       GLuint gl_handle,
-                       GLenum gl_target,
-                       float *subtexture_coords,
-                       float *virtual_coords,
-                       void *user_data)
+                             GLuint gl_handle,
+                             GLenum gl_target,
+                             const float *subtexture_coords,
+                             const float *virtual_coords,
+                             void *user_data)
 {
   TextureSlicedPolygonState *state = user_data;
   GLfloat *v;
@@ -706,14 +703,13 @@ draw_polygon_sub_texture_cb (CoglHandle tex_handle,
   options.layer0_override_texture = gl_handle;
 
   _cogl_material_flush_gl_state (ctx->source_material, &options);
-  _cogl_flush_matrix_stacks ();
 
   GE (glDrawArrays (GL_TRIANGLE_FAN, 0, state->n_vertices));
 }
 
 /* handles 2d-sliced textures with > 1 slice */
 static void
-_cogl_texture_polygon_multiple_primitives (CoglTextureVertex *vertices,
+_cogl_texture_polygon_multiple_primitives (const CoglTextureVertex *vertices,
                                            unsigned int n_vertices,
                                            unsigned int stride,
                                            gboolean use_color)
@@ -766,10 +762,10 @@ _cogl_texture_polygon_multiple_primitives (CoglTextureVertex *vertices,
 }
 
 static void
-_cogl_multitexture_polygon_single_primitive (CoglTextureVertex *vertices,
-                                             guint n_vertices,
-                                             guint n_layers,
-                                             guint stride,
+_cogl_multitexture_polygon_single_primitive (const CoglTextureVertex *vertices,
+                                             unsigned int n_vertices,
+                                             unsigned int n_layers,
+                                             unsigned int stride,
                                              gboolean use_color,
                                              guint32 fallback_layers)
 {
@@ -840,25 +836,23 @@ _cogl_multitexture_polygon_single_primitive (CoglTextureVertex *vertices,
     options.flags |= COGL_MATERIAL_FLUSH_SKIP_GL_COLOR;
   options.fallback_layers = fallback_layers;
   _cogl_material_flush_gl_state (ctx->source_material, &options);
-  _cogl_flush_matrix_stacks ();
 
   GE (glDrawArrays (GL_TRIANGLE_FAN, 0, n_vertices));
 }
 
 void
-cogl_polygon (CoglTextureVertex *vertices,
-              guint              n_vertices,
-	      gboolean           use_color)
+cogl_polygon (const CoglTextureVertex *vertices,
+              unsigned int             n_vertices,
+	      gboolean                 use_color)
 {
   CoglHandle           material;
-  const GList         *layers;
+  const GList         *layers, *tmp;
   int                  n_layers;
-  GList               *tmp;
   gboolean	       use_sliced_polygon_fallback = FALSE;
   guint32              fallback_layers = 0;
   int                  i;
-  gulong               enable_flags;
-  guint                stride;
+  unsigned long        enable_flags;
+  unsigned int         stride;
   gsize                stride_bytes;
   GLfloat             *v;
   int                  prev_n_texcoord_arrays_enabled;
@@ -866,22 +860,31 @@ cogl_polygon (CoglTextureVertex *vertices,
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   _cogl_journal_flush ();
-  cogl_clip_ensure ();
+
+  /* NB: _cogl_framebuffer_flush_state may disrupt various state (such
+   * as the material state) when flushing the clip stack, so should
+   * always be done first when preparing to draw. */
+  _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
 
   material = ctx->source_material;
   layers = cogl_material_get_layers (ctx->source_material);
   n_layers = g_list_length ((GList *)layers);
 
-  for (tmp = (GList *)layers, i = 0; tmp != NULL; tmp = tmp->next, i++)
+  for (tmp = layers, i = 0; tmp != NULL; tmp = tmp->next, i++)
     {
-      CoglHandle   layer = (CoglHandle)tmp->data;
-      CoglHandle   tex_handle = cogl_material_layer_get_texture (layer);
+      CoglHandle layer = tmp->data;
+      CoglHandle tex_handle = cogl_material_layer_get_texture (layer);
 
       /* COGL_INVALID_HANDLE textures will be handled in
        * _cogl_material_flush_layers_gl_state */
       if (tex_handle == COGL_INVALID_HANDLE)
         continue;
 
+      /* Give the texture a chance to know that we're rendering
+         non-quad shaped primitives. If the texture is in an atlas it
+         will be migrated */
+      _cogl_texture_ensure_non_quad_rendering (tex_handle);
+
       if (i == 0 && cogl_texture_is_sliced (tex_handle))
         {
 #if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
@@ -905,6 +908,7 @@ cogl_polygon (CoglTextureVertex *vertices,
                   warning_seen = TRUE;
                 }
             }
+
           use_sliced_polygon_fallback = TRUE;
           n_layers = 1;
 
@@ -916,7 +920,7 @@ cogl_polygon (CoglTextureVertex *vertices,
                 {
                   g_warning ("cogl_texture_polygon does not work for sliced textures "
                              "when the minification and magnification filters are not "
-                             "CGL_NEAREST");
+                             "COGL_MATERIAL_FILTER_NEAREST");
                   warning_seen = TRUE;
                 }
               return;
@@ -970,7 +974,7 @@ cogl_polygon (CoglTextureVertex *vertices,
 
   if (use_color)
     {
-      enable_flags |= COGL_ENABLE_COLOR_ARRAY;
+      enable_flags |= COGL_ENABLE_COLOR_ARRAY | COGL_ENABLE_BLEND;
       GE( glColorPointer (4, GL_UNSIGNED_BYTE,
                           stride_bytes,
                           /* NB: [X,Y,Z,TX,TY...,R,G,B,A,...] */
@@ -978,6 +982,7 @@ cogl_polygon (CoglTextureVertex *vertices,
     }
 
   cogl_enable (enable_flags);
+  _cogl_flush_face_winding ();
 
   GE (glVertexPointer (3, GL_FLOAT, stride_bytes, v));
 
@@ -990,9 +995,10 @@ cogl_polygon (CoglTextureVertex *vertices,
                              /* NB: [X,Y,Z,TX,TY...,R,G,B,A,...] */
                              v + 3 + 2 * i));
     }
-  prev_n_texcoord_arrays_enabled =
-    ctx->n_texcoord_arrays_enabled;
+
+  prev_n_texcoord_arrays_enabled = ctx->n_texcoord_arrays_enabled;
   ctx->n_texcoord_arrays_enabled = n_layers;
+
   for (; i < prev_n_texcoord_arrays_enabled; i++)
     {
       GE (glClientActiveTexture (GL_TEXTURE0 + i));
@@ -1017,578 +1023,3 @@ cogl_polygon (CoglTextureVertex *vertices,
   g_array_set_size (ctx->logged_vertices, 0);
 }
 
-void
-cogl_path_fill (void)
-{
-  cogl_path_fill_preserve ();
-
-  cogl_path_new ();
-}
-
-void
-cogl_path_fill_preserve (void)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  _cogl_journal_flush ();
-  cogl_clip_ensure ();
-
-  if (ctx->path_nodes->len == 0)
-    return;
-
-  _cogl_path_fill_nodes ();
-}
-
-void
-cogl_path_stroke (void)
-{
-  cogl_path_stroke_preserve ();
-
-  cogl_path_new ();
-}
-
-void
-cogl_path_stroke_preserve (void)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  if (ctx->path_nodes->len == 0)
-    return;
-
-  _cogl_journal_flush ();
-  cogl_clip_ensure ();
-
-  _cogl_path_stroke_nodes();
-}
-
-void
-cogl_path_move_to (float x,
-                   float y)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  /* FIXME: handle multiple contours maybe? */
-
-  _cogl_path_add_node (TRUE, x, y);
-
-  ctx->path_start.x = x;
-  ctx->path_start.y = y;
-
-  ctx->path_pen = ctx->path_start;
-}
-
-void
-cogl_path_rel_move_to (float x,
-                       float y)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  cogl_path_move_to (ctx->path_pen.x + x,
-                     ctx->path_pen.y + y);
-}
-
-void
-cogl_path_line_to (float x,
-                   float y)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  _cogl_path_add_node (FALSE, x, y);
-
-  ctx->path_pen.x = x;
-  ctx->path_pen.y = y;
-}
-
-void
-cogl_path_rel_line_to (float x,
-                       float y)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  cogl_path_line_to (ctx->path_pen.x + x,
-                     ctx->path_pen.y + y);
-}
-
-void
-cogl_path_close (void)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  _cogl_path_add_node (FALSE, ctx->path_start.x, ctx->path_start.y);
-  ctx->path_pen = ctx->path_start;
-}
-
-void
-cogl_path_new (void)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  g_array_set_size (ctx->path_nodes, 0);
-}
-
-void
-cogl_path_line (float x_1,
-	        float y_1,
-	        float x_2,
-	        float y_2)
-{
-  cogl_path_move_to (x_1, y_1);
-  cogl_path_line_to (x_2, y_2);
-}
-
-void
-cogl_path_polyline (float *coords,
-	            gint num_points)
-{
-  gint c = 0;
-
-  cogl_path_move_to (coords[0], coords[1]);
-
-  for (c = 1; c < num_points; ++c)
-    cogl_path_line_to (coords[2*c], coords[2*c+1]);
-}
-
-void
-cogl_path_polygon (float *coords,
-	           gint          num_points)
-{
-  cogl_path_polyline (coords, num_points);
-  cogl_path_close ();
-}
-
-void
-cogl_path_rectangle (float x_1,
-                     float y_1,
-                     float x_2,
-                     float y_2)
-{
-  cogl_path_move_to (x_1, y_1);
-  cogl_path_line_to (x_2, y_1);
-  cogl_path_line_to (x_2, y_2);
-  cogl_path_line_to (x_1, y_2);
-  cogl_path_close   ();
-}
-
-static void
-_cogl_path_arc (float center_x,
-	        float center_y,
-                float radius_x,
-                float radius_y,
-                float angle_1,
-                float angle_2,
-                float angle_step,
-                guint        move_first)
-{
-  float a     = 0x0;
-  float cosa  = 0x0;
-  float sina  = 0x0;
-  float px    = 0x0;
-  float py    = 0x0;
-
-  /* Fix invalid angles */
-
-  if (angle_1 == angle_2 || angle_step == 0x0)
-    return;
-
-  if (angle_step < 0x0)
-    angle_step = -angle_step;
-
-  /* Walk the arc by given step */
-
-  a = angle_1;
-  while (a != angle_2)
-    {
-      cosa = cosf (a * (G_PI/180.0));
-      sina = sinf (a * (G_PI/180.0));
-
-      px = center_x + (cosa * radius_x);
-      py = center_y + (sina * radius_y);
-
-      if (a == angle_1 && move_first)
-	cogl_path_move_to (px, py);
-      else
-	cogl_path_line_to (px, py);
-
-      if (G_LIKELY (angle_2 > angle_1))
-        {
-          a += angle_step;
-          if (a > angle_2)
-            a = angle_2;
-        }
-      else
-        {
-          a -= angle_step;
-          if (a < angle_2)
-            a = angle_2;
-        }
-    }
-
-  /* Make sure the final point is drawn */
-
-  cosa = cosf (angle_2 * (G_PI/180.0));
-  sina = sinf (angle_2 * (G_PI/180.0));
-
-  px = center_x + (cosa * radius_x);
-  py = center_y + (sina * radius_y);
-
-  cogl_path_line_to (px, py);
-}
-
-void
-cogl_path_arc (float center_x,
-               float center_y,
-               float radius_x,
-               float radius_y,
-               float angle_1,
-               float angle_2)
-{
-  float angle_step = 10;
-  /* it is documented that a move to is needed to create a freestanding
-   * arc
-   */
-  _cogl_path_arc (center_x,   center_y,
-	          radius_x,   radius_y,
-	          angle_1,    angle_2,
-	          angle_step, 0 /* no move */);
-}
-
-
-void
-cogl_path_arc_rel (float center_x,
-		   float center_y,
-		   float radius_x,
-		   float radius_y,
-		   float angle_1,
-		   float angle_2,
-		   float angle_step)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  _cogl_path_arc (ctx->path_pen.x + center_x,
-	          ctx->path_pen.y + center_y,
-	          radius_x,   radius_y,
-	          angle_1,    angle_2,
-	          angle_step, 0 /* no move */);
-}
-
-void
-cogl_path_ellipse (float center_x,
-                   float center_y,
-                   float radius_x,
-                   float radius_y)
-{
-  float angle_step = 10;
-
-  /* FIXME: if shows to be slow might be optimized
-   * by mirroring just a quarter of it */
-
-  _cogl_path_arc (center_x, center_y,
-	          radius_x, radius_y,
-	          0, 360,
-	          angle_step, 1 /* move first */);
-
-  cogl_path_close();
-}
-
-void
-cogl_path_round_rectangle (float x_1,
-                           float y_1,
-                           float x_2,
-                           float y_2,
-                           float radius,
-                           float arc_step)
-{
-  float inner_width = x_2 - x_1 - radius * 2;
-  float inner_height = y_2 - y_1 - radius * 2;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  cogl_path_move_to (x_1, y_1 + radius);
-  cogl_path_arc_rel (radius, 0,
-		     radius, radius,
-		     180,
-		     270,
-		     arc_step);
-
-  cogl_path_line_to       (ctx->path_pen.x + inner_width,
-                           ctx->path_pen.y);
-  cogl_path_arc_rel       (0, radius,
-			   radius, radius,
-			   -90,
-			   0,
-			   arc_step);
-
-  cogl_path_line_to       (ctx->path_pen.x,
-                           ctx->path_pen.y + inner_height);
-
-  cogl_path_arc_rel       (-radius, 0,
-			   radius, radius,
-			   0,
-			   90,
-			   arc_step);
-
-  cogl_path_line_to       (ctx->path_pen.x - inner_width,
-                           ctx->path_pen.y);
-  cogl_path_arc_rel       (0, -radius,
-			   radius, radius,
-			   90,
-			   180,
-			   arc_step);
-
-  cogl_path_close ();
-}
-
-
-static void
-_cogl_path_bezier3_sub (CoglBezCubic *cubic)
-{
-  CoglBezCubic   cubics[_COGL_MAX_BEZ_RECURSE_DEPTH];
-  CoglBezCubic  *cleft;
-  CoglBezCubic  *cright;
-  CoglBezCubic  *c;
-  floatVec2  dif1;
-  floatVec2  dif2;
-  floatVec2  mm;
-  floatVec2  c1;
-  floatVec2  c2;
-  floatVec2  c3;
-  floatVec2  c4;
-  floatVec2  c5;
-  gint           cindex;
-
-  /* Put first curve on stack */
-  cubics[0] = *cubic;
-  cindex    =  0;
-
-  while (cindex >= 0)
-    {
-      c = &cubics[cindex];
-
-
-      /* Calculate distance of control points from their
-       * counterparts on the line between end points */
-      dif1.x = (c->p2.x * 3) - (c->p1.x * 2) - c->p4.x;
-      dif1.y = (c->p2.y * 3) - (c->p1.y * 2) - c->p4.y;
-      dif2.x = (c->p3.x * 3) - (c->p4.x * 2) - c->p1.x;
-      dif2.y = (c->p3.y * 3) - (c->p4.y * 2) - c->p1.y;
-
-      if (dif1.x < 0)
-        dif1.x = -dif1.x;
-      if (dif1.y < 0)
-        dif1.y = -dif1.y;
-      if (dif2.x < 0)
-        dif2.x = -dif2.x;
-      if (dif2.y < 0)
-        dif2.y = -dif2.y;
-
-
-      /* Pick the greatest of two distances */
-      if (dif1.x < dif2.x) dif1.x = dif2.x;
-      if (dif1.y < dif2.y) dif1.y = dif2.y;
-
-      /* Cancel if the curve is flat enough */
-      if (dif1.x + dif1.y <= 1.0 ||
-	  cindex == _COGL_MAX_BEZ_RECURSE_DEPTH-1)
-	{
-	  /* Add subdivision point (skip last) */
-	  if (cindex == 0)
-            return;
-
-	  _cogl_path_add_node (FALSE, c->p4.x, c->p4.y);
-
-	  --cindex;
-
-          continue;
-	}
-
-      /* Left recursion goes on top of stack! */
-      cright = c; cleft = &cubics[++cindex];
-
-      /* Subdivide into 2 sub-curves */
-      c1.x = ((c->p1.x + c->p2.x) / 2);
-      c1.y = ((c->p1.y + c->p2.y) / 2);
-      mm.x = ((c->p2.x + c->p3.x) / 2);
-      mm.y = ((c->p2.y + c->p3.y) / 2);
-      c5.x = ((c->p3.x + c->p4.x) / 2);
-      c5.y = ((c->p3.y + c->p4.y) / 2);
-
-      c2.x = ((c1.x + mm.x) / 2);
-      c2.y = ((c1.y + mm.y) / 2);
-      c4.x = ((mm.x + c5.x) / 2);
-      c4.y = ((mm.y + c5.y) / 2);
-
-      c3.x = ((c2.x + c4.x) / 2);
-      c3.y = ((c2.y + c4.y) / 2);
-
-      /* Add left recursion to stack */
-      cleft->p1 = c->p1;
-      cleft->p2 = c1;
-      cleft->p3 = c2;
-      cleft->p4 = c3;
-
-      /* Add right recursion to stack */
-      cright->p1 = c3;
-      cright->p2 = c4;
-      cright->p3 = c5;
-      cright->p4 = c->p4;
-    }
-}
-
-void
-cogl_path_curve_to (float x_1,
-                    float y_1,
-                    float x_2,
-                    float y_2,
-                    float x_3,
-                    float y_3)
-{
-  CoglBezCubic cubic;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  /* Prepare cubic curve */
-  cubic.p1 = ctx->path_pen;
-  cubic.p2.x = x_1;
-  cubic.p2.y = y_1;
-  cubic.p3.x = x_2;
-  cubic.p3.y = y_2;
-  cubic.p4.x = x_3;
-  cubic.p4.y = y_3;
-
-  /* Run subdivision */
-  _cogl_path_bezier3_sub (&cubic);
-
-  /* Add last point */
-  _cogl_path_add_node (FALSE, cubic.p4.x, cubic.p4.y);
-  ctx->path_pen = cubic.p4;
-}
-
-void
-cogl_path_rel_curve_to (float x_1,
-                        float y_1,
-                        float x_2,
-                        float y_2,
-                        float x_3,
-                        float y_3)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  cogl_path_curve_to (ctx->path_pen.x + x_1,
-                      ctx->path_pen.y + y_1,
-                      ctx->path_pen.x + x_2,
-                      ctx->path_pen.y + y_2,
-                      ctx->path_pen.x + x_3,
-                      ctx->path_pen.y + y_3);
-}
-
-
-/* If second order beziers were needed the following code could
- * be re-enabled:
- */
-#if 0
-
-static void
-_cogl_path_bezier2_sub (CoglBezQuad *quad)
-{
-  CoglBezQuad     quads[_COGL_MAX_BEZ_RECURSE_DEPTH];
-  CoglBezQuad    *qleft;
-  CoglBezQuad    *qright;
-  CoglBezQuad    *q;
-  floatVec2   mid;
-  floatVec2   dif;
-  floatVec2   c1;
-  floatVec2   c2;
-  floatVec2   c3;
-  gint            qindex;
-
-  /* Put first curve on stack */
-  quads[0] = *quad;
-  qindex   =  0;
-
-  /* While stack is not empty */
-  while (qindex >= 0)
-    {
-
-      q = &quads[qindex];
-
-      /* Calculate distance of control point from its
-       * counterpart on the line between end points */
-      mid.x = ((q->p1.x + q->p3.x) / 2);
-      mid.y = ((q->p1.y + q->p3.y) / 2);
-      dif.x = (q->p2.x - mid.x);
-      dif.y = (q->p2.y - mid.y);
-      if (dif.x < 0) dif.x = -dif.x;
-      if (dif.y < 0) dif.y = -dif.y;
-
-      /* Cancel if the curve is flat enough */
-      if (dif.x + dif.y <= 1.0 ||
-          qindex == _COGL_MAX_BEZ_RECURSE_DEPTH - 1)
-	{
-	  /* Add subdivision point (skip last) */
-	  if (qindex == 0) return;
-	  _cogl_path_add_node (FALSE, q->p3.x, q->p3.y);
-	  --qindex; continue;
-	}
-
-      /* Left recursion goes on top of stack! */
-      qright = q; qleft = &quads[++qindex];
-
-      /* Subdivide into 2 sub-curves */
-      c1.x = ((q->p1.x + q->p2.x) / 2);
-      c1.y = ((q->p1.y + q->p2.y) / 2);
-      c3.x = ((q->p2.x + q->p3.x) / 2);
-      c3.y = ((q->p2.y + q->p3.y) / 2);
-      c2.x = ((c1.x + c3.x) / 2);
-      c2.y = ((c1.y + c3.y) / 2);
-
-      /* Add left recursion onto stack */
-      qleft->p1 = q->p1;
-      qleft->p2 = c1;
-      qleft->p3 = c2;
-
-      /* Add right recursion onto stack */
-      qright->p1 = c2;
-      qright->p2 = c3;
-      qright->p3 = q->p3;
-    }
-}
-
-void
-cogl_path_curve2_to (float x_1,
-                     float y_1,
-                     float x_2,
-                     float y_2)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  CoglBezQuad quad;
-
-  /* Prepare quadratic curve */
-  quad.p1 = ctx->path_pen;
-  quad.p2.x = x_1;
-  quad.p2.y = y_1;
-  quad.p3.x = x_2;
-  quad.p3.y = y_2;
-
-  /* Run subdivision */
-  _cogl_path_bezier2_sub (&quad);
-
-  /* Add last point */
-  _cogl_path_add_node (FALSE, quad.p3.x, quad.p3.y);
-  ctx->path_pen = quad.p3;
-}
-
-void
-cogl_rel_curve2_to (float x_1,
-                    float y_1,
-                    float x_2,
-                    float y_2)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  cogl_path_curve2_to (ctx->path_pen.x + x_1,
-                       ctx->path_pen.y + y_1,
-                       ctx->path_pen.x + x_2,
-                       ctx->path_pen.y + y_2);
-}
-#endif
