@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * Authors:
  *  Matthew Allum  <mallum@openedhand.com>
@@ -841,15 +841,14 @@ _cogl_texture_2d_sliced_slices_create (CoglTexture2DSliced *tex_2ds,
         {
           x_span = &g_array_index (tex_2ds->slice_x_spans, CoglSpan, x);
 
-          COGL_NOTE (TEXTURE, "CREATE SLICE (%d,%d)\tsize (%d,%d)",
+          COGL_NOTE (SLICING, "CREATE SLICE (%d,%d)\tsize (%d,%d)",
                      x, y,
                      x_span->size - x_span->waste,
                      y_span->size - y_span->waste);
 
           /* Setup texture parameters */
-          GE( _cogl_texture_driver_bind (tex_2ds->gl_target,
-                                         gl_handles[y * n_x_slices + x],
-                                         gl_intformat) );
+          GE( glBindTexture (tex_2ds->gl_target,
+                             gl_handles[y * n_x_slices + x] ) );
 
           _cogl_texture_driver_try_setting_gl_border_color (tex_2ds->gl_target,
                                                             transparent_color);
@@ -1076,11 +1075,8 @@ _cogl_texture_2d_sliced_new_from_foreign (GLuint           gl_handle,
                                           CoglPixelFormat  format)
 {
   /* NOTE: width, height and internal format are not queriable
-     in GLES, hence such a function prototype. However, for
-     OpenGL they are still queried from the texture for improved
-     robustness and for completeness in case one day GLES gains
-     support for them.
-     */
+   * in GLES, hence such a function prototype.
+   */
 
   GLenum               gl_error = 0;
   GLboolean            gl_istexture;
@@ -1132,19 +1128,16 @@ _cogl_texture_2d_sliced_new_from_foreign (GLuint           gl_handle,
   GE( glGetTexLevelParameteriv (gl_target, 0,
                                 GL_TEXTURE_INTERNAL_FORMAT,
                                 &gl_int_format) );
+#endif
 
-
-  GE( glGetTexLevelParameteriv (gl_target, 0,
-                                GL_TEXTURE_WIDTH,
-                                &gl_width) );
-
-  GE( glGetTexLevelParameteriv (gl_target, 0,
-                                GL_TEXTURE_HEIGHT,
-                                &gl_height) );
-#else
+  /* Note: We always trust the given width and height without querying
+   * the texture object because the user may be creating a Cogl
+   * texture for a texture_from_pixmap object where glTexImage2D may
+   * not have been called and the texture_from_pixmap spec doesn't
+   * clarify that it is reliable to query back the size from OpenGL.
+   */
   gl_width = width + x_pot_waste;
   gl_height = height + y_pot_waste;
-#endif
 
   GE( glGetTexParameteriv (gl_target,
                            GL_GENERATE_MIPMAP,
